export function addEnums(ctx: any): void;
export type PexContextOptions = {
    gl?: WebGLRenderingContext | WebGL2RenderingContext;
    width?: number;
    height?: number;
    pixelRatio?: number;
    type?: "webgl" | "webgl2";
    debug?: boolean;
};
/**
 * All resources are plain js object and once constructed their properties can be accessed directly.
 * Please note those props are read only. To set new values or upload new data to GPU see [updating resources]{@link context ~update}.
 */
export type PexResource = {
    name: string;
};
export type PexCommand = {
    pass: import("./pass.js").PassOptions;
    pipeline: import("./pipeline.js").PipelineOptions;
    /**
     * vertex attributes, map of `attibuteName: ctx.VertexBuffer`   or `attributeName: { buffer: VertexBuffer, offset: number, stride: number, divisor: number }`
     */
    attributes: any;
    /**
     * indices, `ctx.IndexBuffer` or `{ buffer: IndexBuffer, offset: number, stride: number }`
     */
    indices: any;
    /**
     * number of vertices to draw
     */
    count: number;
    /**
     * number instances to draw
     */
    instances: number;
    /**
     * shader uniforms, map of `name: value`
     */
    uniforms: any;
    /**
     * drawing viewport bounds
     */
    viewport: Viewport;
    /**
     * scissor test bounds
     */
    scissor: Viewport;
};
export type PexContextSetOptions = {
    width?: number;
    height?: number;
    pixelRatio?: number;
};
/**
 * [x, y, w, h]
 */
export type Viewport = number[];
/**
 * [r, g, b, a]
 */
export type Color = number[];
